<?php
/**
 * @name		WebTotem Security
 * @package		pkg_wtotem
 * @copyright	Copyright (C) 2023. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 * @author		WebTotem team
 */

Namespace WebTotem;

// No direct access
defined('_JEXEC') or die;
defined('WEBTOTEM_INIT') or die;

/**
 * Agent Manager library.
 *
 * Agent Manager is needed to create AM file,
 *   and to check whether am and other agents are installed.
 * The AM file creates WAV and AV files,
 *   and transmits information to the Web Totem platform and back.
 */
class AgentManager extends Helper {

	/**
	 * AM file install.
	 *
	 * Create AM file in the root of the site
	 *   and save the data about it in the DB.
	 *
	 * @return bool
	 *   TRUE if the AM file is successfully added.
	 */
	public static function amInstall() {
		try {
			self::removeAgents();

			$host  = API::siteInfo();

			$files = self::getAgentsFiles( $host['id'] );

			if ( $files['amFilename'] ) {

				// Download file.
				$result = self::downloadFile($files);

				// If the file is downloaded, then we write the data to the DB.
				if ( $result ) {
					Option::setOptions( [
						'am_installed' => true,
						'am_file'      => $files['amFilename'],
						'waf_file'     => $files['wafFilename'],
						'av_file'      => $files['avFilename'],
					] );

					self::generateMarkerFile();

					$message = \JText::_('WTOTEM_AM_SUCCESSFULLY_INSTALLED');
					Option::setNotification( 'success', $message );
				}
				else {
					$message = sprintf( \JText::_('WTOTEM_CHECK_FOLDER_WRITE_PERMISSION'), WTOTEM_SITE_ROOT)
						. sprintf(\JText::_('WTOTEM_READ_MORE_HERE'),
						'https://docs.wtotem.com/agent-setup#it-additional-recommendations');
					Option::setNotification( 'error', $message );

					return FALSE;
				}
			}

		} catch ( \Exception $e ) {
			Option::setNotification( 'error', $e->getMessage() );

			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Get the AM file download link and agents (AV, WAF) files name.
	 *
	 * @param string $host_id
	 *   Host id on WebTotem.
	 *
	 * @return array
	 *   Agent Manager download link, names of agents
	 */
	public static function getAgentsFiles( $host_id ) {
		$result = API::getAgentsFiles( $host_id );
		if ( ! $result  ) {
			$file = [ 'amFilename' => NULL, 'downloadLink' => NULL ];

			$message = \JText::_('WTOTEM_ERROR_GENERATING_THE_AM_FILE');
			Option::setNotification( 'error', $message );
		} else {
			$file = $result;
		}

		return $file;
	}

	/**
	 * Check the existence of the service file and the record in the DB.
	 *
	 * @param string $service
	 *   Service short name (am, av, waf).
	 *
	 * @return array
	 *   Service file data (installed status, file exist status, file name).
	 */
	public static function checkInstalledService( $service ) {
		$file = Option::getOption( $service . "_file" );

		return [
			'option_status' => Option::getOption( $service . "_installed" ),
			'file_status'   => ( (bool) $file ) && is_file( WTOTEM_SITE_ROOT . $file ),
			'file_name'     => $file,
		];
	}

	/**
	 * Remove all agents files and folders. Clear agents options.
	 *
	 * @return bool
	 *   Returns TRUE if agent files
	 *   and folders have been successfully deleted.
	 */
	public static function removeAgents() {
		// Deleting all agent records from the database.
		Option::clearOptions( [
			'am_installed',
			'waf_installed',
			'av_installed',
			'am_file',
			'waf_file',
			'av_file',
		] );

		$list = scandir( WTOTEM_SITE_ROOT );

		foreach ( $list as $item ) {
			$target_item = WTOTEM_SITE_ROOT . $item;
			$pattern = '/([a-zA-Z0-9_]{64}.av.php)|([a-zA-Z0-9_]{64}.am.php)|([a-zA-Z0-9_]{64}.waf.php)|(\.wtotem_[a-zA-Z0-9_]{12,16})/';

			if ( preg_match( $pattern, $target_item ) ) {
				if(is_dir($target_item)){
					if(\JFolder::exists($target_item) ) {
						\JFolder::delete($target_item);
					}
				} else{
					if(\JFile::exists($target_item) ) {
						\JFile::delete($target_item);
					}
				}
			}

		}

		return TRUE;
	}

	/**
	 * @param $url
	 *   Link from where to download the file.
	 * @param $path
	 *   Path where to save the file.
	 *
	 * @return bool
	 *   If the file is saved successfully, it returns true.
	 */
	private static function downloadFile($files) {

		// Create an instance of a default JHttp object.
		$http = \JHttpFactory::getHttp();

		// Invoke the GET request.
		$response = $http->get($files['downloadLink']);

		$path = WTOTEM_SITE_ROOT . $files['amFilename'];

		if ($response->code < 200) {
			Option::setNotification('error', \JText::_('WTOTEM_COULD_NOT_DOWNLOAD_FILE'));
			return FALSE;
		}

		if(!empty($response->body)){
			return file_put_contents($path, $response->body);
		} else {
			$message = \JText::_('WTOTEM_API_RESPONSE_BODY_IS_EMPTY');
			Option::setNotification( 'error', $message );
		}

		return FALSE;

	}

	/**
	 * Generate the file that indicates that a WAF connection is being used through the plugin.
	 */
	public static function generateMarkerFile() {

		if($am_filename = Option::getOption('am_file')) {
      $content = '<?php exit(); ?>' . $am_filename;
      $file_path = WTOTEM_PATH . '/generate.php';
			if ( ! file_exists($file_path) or file_get_contents($file_path) != $content) {
				if ( ! file_put_contents( $file_path, $content ) ) {
					$message = sprintf( \JText::_('WTOTEM_CHECK_FOLDER_WRITE_PERMISSION'), WTOTEM_SITE_ROOT)
						. sprintf(\JText::_('WTOTEM_READ_MORE_HERE'),
							'https://docs.wtotem.com/agent-setup#it-additional-recommendations');
					Option::setNotification( 'error', $message );
				}
			}
		}

	}

	/**
	 * Check if the plugin version has changed.
	 */
	public static function checkVersion(){
		if(Option::isActivated()){
			// Get version of the plugin that was previously installed.
			$version = Option::getOption('plugin_version');

			if ($version == WEBTOTEM_VERSION) {
				return;
			}

			Option::setOptions(['plugin_version' => WEBTOTEM_VERSION]);

			// Generate the file that indicates that a WAF connection is being used through the plugin.
			self::generateMarkerFile();
		}

	}

}
