<?php
/**
 * @name		WebTotem Security
 * @package		pkg_wtotem
 * @copyright	Copyright (C) 2023. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 * @author		WebTotem team
 */


Namespace WebTotem;

use WebTotem\Helper;

// No direct access
defined('_JEXEC') or die;
defined('WEBTOTEM_INIT') or die;

class Ajax {

	/**
	 * Activation plugin.
	 *
	 * @return void
	 */
	public static function activation() {

		if (Helper::postRequest('ajax_action') != 'activation') {
			return;
		}

		if ($api_key = Helper::postRequest('api_key')) {

			$result = API::auth($api_key);

			if ($result == 'success') {
				$link = WTOTEM_URL . '&task=activation';

				echo json_encode([
						'notifications' => self::notifications(),
						'link' => $link,
						'success' => true,
				]);
			} else {

				echo json_encode([
						'notifications' => self::notifications(),
						'success' => false,
				]);

			}
		}

	}

	/**
	 * The process of installing agents (WAF, AV) on the main page.
	 *
	 * @return void
	 */
	public static function agentsInstallation()
	{

		if (Helper::postRequest('ajax_action') != 'agents_installation') {
			return;
		}

		$av_installed = Option::getOption('av_installed');
		$waf_installed = Option::getOption('waf_installed');

		// Check if the agents are installed.
		if ($av_installed and $waf_installed) {
			$agents_statuses = [
				'process_statuses' => [
					'av' => 'installed',
					'waf' => 'installed',
				],
			];
		} else {
			// If not installed, then request statuses from the Helper API.
			$host = API::siteInfo();
			$data = API::getAgentsStatusesFromAPI($host['id']);

			$agents_statuses = [
				'av' => $data['av']['status'],
				'waf' => $data['waf']['status'],
			];

			$agents_statuses = Helper::getAgentsStatuses($agents_statuses);
		}

		$build[] = [
			'variables' => [
				'process_status' => $agents_statuses['process_statuses'],
			],
			'template' => 'agents_installation',
		];

		$status = [
			'av' => $agents_statuses['process_statuses']['av'] == 'installed',
			'waf' => $agents_statuses['process_statuses']['waf'] == 'installed',
		];

		Option::setOptions([
			'av_installed' => $status['av'],
			'waf_installed' => $status['waf'],
		]);

		$template = new Template();
		$agents = $template->arrayRender($build);

		echo json_encode([
			'success' => true,
			'notifications' => self::notifications(),
			'agents' => $agents,
			'agents_statuses' => $status['av'] && $status['waf'],
		]);
	}

	/**
	 * Reinstall agents.
	 *
	 * @return void
	 */
	public static function reinstallAgents()
	{

		if (Helper::postRequest('ajax_action') != 'reinstall_agents') {
			return;
		}

		AgentManager::amInstall();

		$response['success'] = true;
		$response['redirect_link'] = \JRoute::_(WTOTEM_URL . '&task=dashboard');
		echo json_encode($response);

	}

	/**
	 * Deleting plugin activation data and redirecting to the activation page.
	 *
	 * @return void
	 */
	public static function logout()
	{

		if (Helper::postRequest('ajax_action') !== 'logout') {
			return;
		}

		Option::logout();

		$response['success'] = true;
		$response['redirect_link'] = \JRoute::_(WTOTEM_URL . '&task=activation');
		echo json_encode($response);

	}

	/**
	 * Creating a modal window.
	 *
	 * @return void
	 */
	public static function popup()
	{

		if (Helper::postRequest('ajax_action') !== 'popup') {
			return;
		}

		$action = Helper::postRequest('popup_action');
		$template = new Template();

		if ($action) {
			switch ($action) {
				case 'reinstall_agents':

					$uri = \JFactory::getURI();
					$host = $uri->toString(array('host'));

					$build[] = [
						'variables' => [
							'message' => sprintf(\JText::_('WTOTEM_SOME_SCANNING_DATA_BE_DELETED'), $host),
							'action' => 'reinstall_agents',
						],
						'template' => 'popup',
					];
					break;

				case 'logout':
					$build[] = [
						'variables' => [
							'message' => \JText::_('WTOTEM_ARE_YOU_SURE_CHANGE_API_KEY'),
							'action' => 'logout',
						],
						'template' => 'popup',
					];
					break;
			}

			echo json_encode([
				'success' => true,
				'content' => $template->arrayRender($build),
			]);
			return;
		}

		echo json_encode([
			'success' => false,
		]);

	}

	/**
	 * Request to update charts with parameters.
	 *
	 * @return void
	 */
	public static function chart()
	{

		if (Helper::postRequest('ajax_action') !== 'chart') {
			return;
		}

		$template = new Template();

		$days = (integer)Helper::postRequest('days');
		$service = Helper::postRequest('service');

		$host = API::siteInfo();

		switch ($service) {
			case 'waf':

				Option::setSessionOptions(['firewall_period' => $days]);

				// Firewall chart.
				$data = API::getFirewallChart($host['id'], $days);
				$chart = Helper::generateWafChart($data['chart']);

				$_chart[] = [
					'variables' => [
						'days' => $days,
						'chart' => $chart['chart'],
					],
					'template' => 'firewall_chart',
				];

				// Firewall logs.
				$data = API::getFirewall($host['id'], 10, NULL, $days);
				$firewall = $data['firewall'];

				$waf_logs[] = [
					'variables' => [
						'logs' => Helper::wafLogs($firewall['logs']['edges']),
					],
					'template' => 'firewall_logs',
				];

				// Firewall stats.
				$waf_stats[] = [
					'variables' => [
						'is_waf_training' => Helper::isWafTraining($data['agentManager']['createdAt']),
						'all_attacks' => $chart['count_attacks'],
						'blocking' => $chart['count_blocks'],
						'not_blocking' => $chart['count_attacks'] - $chart['count_blocks'],
						'most_attacks' => Helper::getMostAttacksData($firewall['map']),
					],
					'template' => 'firewall_stats',
				];

				Option::setSessionOptions([
					'firewall_cursor' => $firewall['logs']['pageInfo']['endCursor'],
				]);

				$has_next_page = $firewall['logs']['pageInfo']['hasNextPage'];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'waf_logs' => $template->arrayRender($waf_logs),
					'waf_stats' => $template->arrayRender($waf_stats),
					'has_next_page' => $has_next_page,
					'service' => 'waf',
				];

				break;

			case 'cpu':
				Option::setSessionOptions(['cpu_period' => $days]);

				$data = API::getServerStatusData($host['id'], $days);
				$chart = Helper::generateChart($data['cpuChart'], $days);

				$_chart[] = [
					'variables' => [
						'days' => $days,
						'chart' => $chart,
					],
					'template' => 'cpu_chart',
				];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'service' => 'cpu',
				];

				break;

			case 'ram':
				Option::setSessionOptions(['ram_period' => $days]);

				$data = API::getServerStatusData($host['id'], $days);
				$chart = Helper::generateChart($data['ramChart'], $days);

				$_chart[] = [
					'variables' => [
						'days' => $days,
						'chart' => $chart,
					],
					'template' => 'ram_chart',
				];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'service' => 'ram',
				];

				break;

			case 'map':
				$data = API::getFirewallChart($host['id'], $days);
				$chart = Helper::generateAttacksMapChart($data['map']);
				$world_map_json = WTOTEM_MEDIA_URI . '/assets/js/world_map.json';

				$_chart[] = [
					'variables' => [
						'attacks_map' => $chart,
						'world_map_json' => $world_map_json,
					],
					'template' => 'map_chart',
				];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'service' => 'map',
				];

				break;

		}

		if ($service) {
			$response['success'] = true;
			$response['notifications'] = self::notifications();
			echo json_encode($response);
		}

	}

	/**
	 * Data lazy load.
	 * @return void
	 */
	public static function lazyLoad()
	{


		if (Helper::postRequest('ajax_action') !== 'lazy_load') {
			return;
		}

		$template = new Template();

		$service = Helper::postRequest('service');

		$host = API::siteInfo();

		switch ($service) {
			case 'firewall':
				$cursor = Option::getSessionOption('firewall_cursor') ?: NULL;
				$period = Option::getSessionOption('firewall_period') ?: 365;
				$data = API::getFirewall($host['id'], 10, $cursor, $period);
				$service_data = $data['firewall'];
				$has_next_page = $service_data['logs']['pageInfo']['hasNextPage'];

				Option::setSessionOptions([
					'firewall_cursor' => $service_data['logs']['pageInfo']['endCursor'],
				]);

				// Firewall logs.
				$build[] = [
					'variables' => [
						'logs' => Helper::wafLogs($service_data['logs']['edges']),
					],
					'template' => 'firewall_logs',
				];

				break;

			case 'antivirus':
				$cursor = Option::getSessionOption('antivirus_cursor') ?: NULL;
				$event = Option::getSessionOption('antivirus_event') ?: NULL;
				$permissions = Option::getSessionOption('antivirus_permissions') ?: NULL;

				$params = [
					'host_id' => $host['id'],
					'limit' => 10,
					'days' => 365,
					'cursor' => $cursor,
					'event' => $event,
					'permissions' => $permissions,
				];

				$data = API::getAntivirus($params);
				$has_next_page = $data['log']['pageInfo']['hasNextPage'];

				Option::setSessionOptions([
					'antivirus_cursor' => $data['log']['pageInfo']['endCursor'],
				]);

				// Antivirus logs.
				$build[] = [
					'variables' => [
						'logs' => Helper::getAntivirusLogs($data['log']['edges']),
					],
					'template' => 'antivirus_logs',
				];

				break;

			case 'reports':
				$cursor = Option::getSessionOption('reports_cursor') ?: NULL;

				$data = API::getAllReports($host['id'], 10, $cursor);
				$has_next_page = $data['pageInfo']['hasNextPage'];

				Option::setSessionOptions([
					'reports_cursor' => $data['pageInfo']['endCursor'],
				]);

				// Reports.
				$build[] = [
					'variables' => [
						"reports" => Helper::getReports($data['edges']),
						"has_next_page" => $data['pageInfo']['hasNextPage'],
					],
					'template' => 'reports_list',
				];

				break;

			case 'reports_m':
				$cursor = Option::getSessionOption('reports_m_cursor') ?: NULL;

				$data = API::getAllReports($host['id'], 10, $cursor);
				$has_next_page = $data['pageInfo']['hasNextPage'];

				Option::setSessionOptions([
					'reports_m_cursor' => $data['pageInfo']['endCursor'],
				]);

				// Reports mobile.
				$build[] = [
					'variables' => [
						"reports" => Helper::getReports($data['edges']),
						"has_next_page" => $data['pageInfo']['hasNextPage'],
					],
					'template' => 'reports_list_mobile',
				];

				break;
		}

		if ($service) {

			echo json_encode([
				'success' => true,
				'content' => $template->arrayRender($build),
				'has_next_page' => $has_next_page,
				'notifications' => self::notifications(),
			]);
		}
	}

	/**
	 * Add date filter.
	 *
	 * @return void
	 */
	public static function wafDateFilter()
	{

		if (Helper::postRequest('ajax_action') !== 'waf_date_filter') {
			return;
		}

		$template = new Template();

		$date_from = Helper::postRequest('date_from');

		$period = explode(" to ", $date_from);
		Option::setSessionOptions(['firewall_period' => $period]);

		$host = API::siteInfo();

		// Firewall logs.
		$data = API::getFirewall($host['id'], 10, NULL, $period);
		$firewall = $data['firewall'];

		$waf_logs[] = [
			'variables' => [
				'logs' => Helper::wafLogs($firewall['logs']['edges']),
			],
			'template' => 'firewall_logs',
		];

		// Firewall chart.
		$data = API::getFirewallChart($host['id'], $period);
		$chart = Helper::generateWafChart($data['chart']);

		$_chart[] = [
			'variables' => [
				'days' => $chart['days'],
				'chart' => $chart['chart'],
			],
			'template' => 'firewall_chart',
		];

		// Firewall stats.
		$waf_stats[] = [
			'variables' => [
				'is_waf_training' => Helper::isWafTraining($data['agentManager']['createdAt']),
				'all_attacks' => $chart['count_attacks'],
				'blocking' => $chart['count_blocks'],
				'not_blocking' => $chart['count_attacks'] - $chart['count_blocks'],
				'most_attacks' => Helper::getMostAttacksData($firewall['map']),
			],
			'template' => 'firewall_stats',
		];

		Option::setSessionOptions([
			'firewall_cursor' => $firewall['logs']['pageInfo']['endCursor'],
		]);

		$has_next_page = $firewall['logs']['pageInfo']['hasNextPage'];

		$response = [
			'success' => true,
			'chart' => $template->arrayRender($_chart),
			'waf_logs' => $template->arrayRender($waf_logs),
			'waf_stats' => $template->arrayRender($waf_stats),
			'has_next_page' => $has_next_page,
			'notifications' => self::notifications(),
		];

		echo json_encode($response);
	}


	/**
	 * Request to restart re-scan and receive antivirus data.
	 *
	 * @return void
	 */
	public static function antivirus()
	{

		if (Helper::postRequest('ajax_action') !== 'antivirus') {
			return;
		}

		$action = Helper::postRequest('av_action');

		$host = API::siteInfo();

		switch ($action) {
			case 'rescan':
				$response = API::forceCheck($host['id'], 'av');

				if (!isset($response['errors'])) {
					$data = API::getAntivirusLastTest($host['id']);
					$response['last_scan'] = Helper::dateFormatter($data['lastTest']['time']);
				}
				break;

			case 'download_report':
				$response = API::avExport($host['id']);
				if (!isset($response['errors'])) {
					$response['doc_link'] = $response['data']['auth']['sites']['av']['export'];
				}
				break;

			case 'filter':

				$file_status = Helper::postRequest('file_status');
				$permission = filter_var(Helper::postRequest('permission'), FILTER_VALIDATE_BOOLEAN);

				Option::setSessionOptions([
					'antivirus_permissions' => $permission,
					'antivirus_event' => $file_status,
				]);

				$params = [
					'host_id' => $host['id'],
					'limit' => 10,
					'days' => 365,
					'cursor' => NULL,
					'event' => $file_status,
					'permissions' => $permission,
				];

				$data = API::getAntivirus($params);
				$has_next_page = $data['log']['pageInfo']['hasNextPage'];

				Option::setSessionOptions([
					'antivirus_cursor' => $data['log']['pageInfo']['endCursor'],
				]);

				// Antivirus logs.
				$build[] = [
					'variables' => [
						'logs' => Helper::getAntivirusLogs($data['log']['edges']),
					],
					'template' => 'antivirus_logs',
				];

				$template = new Template();
				$response = [
					'logs' => $template->arrayRender($build),
					'has_next_page' => $has_next_page,
				];

				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();

		echo json_encode($response);
	}

	/**
	 * Request to add a file to quarantine.
	 *
	 * @return void
	 */
	public static function quarantine()
	{
		if (Helper::postRequest('ajax_action') !== 'quarantine') {
			return;
		}

		$action = Helper::postRequest('quarantine_action');
		$id_or_path = Helper::postRequest('id_or_path');

		$host = API::siteInfo();
		$response = [];

		switch ($action) {
			case 'add':
				$api_response = API::moveToQuarantine($host['id'], $id_or_path);
				break;

			case 'remove':
				$api_response = API::moveFromQuarantine($id_or_path);
				break;
		}

		if (!isset($api_response['errors'])) {

			$quarantine_logs = API::getQuarantineList($host['id']);
			$quarantine_count = count($quarantine_logs);

			// Quarantine logs.
			$quarantine[] = [
				'variables' => [
					"logs" => Helper::getQuarantineLogs($quarantine_logs),
					"count" => $quarantine_count,
				],
				'template' => 'quarantine',
			];

			$cursor = Option::getSessionOption('antivirus_cursor') ?: NULL;
			$event = Option::getSessionOption('antivirus_event') ?: NULL;
			$permissions = Option::getSessionOption('antivirus_permissions') ?: NULL;

			$params = [
				'host_id' => $host['id'],
				'limit' => 10,
				'days' => 365,
				'cursor' => $cursor,
				'event' => $event,
				'permissions' => $permissions,
			];

			$data = API::getAntivirus($params);
			Cache::setData(['getAntivirus' => $data], $host['id']);

			$has_next_page = $data['log']['pageInfo']['hasNextPage'];

			Option::setSessionOptions([
				'antivirus_cursor' => $data['log']['pageInfo']['endCursor'],
			]);

			// Antivirus logs.
			$antivirus_logs[] = [
				'variables' => [
					'logs' => Helper::getAntivirusLogs($data['log']['edges']),
				],
				'template' => 'antivirus_logs',
			];


			$template = new Template();
			$response = [
				'antivirus_logs' => $template->arrayRender($antivirus_logs),
				'quarantine' => $template->arrayRender($quarantine),
				'has_next_page' => $has_next_page,
			];

		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();

		echo json_encode($response);

	}

	/**
	 * Request to add or remove a port to the ignore list.
	 *
	 * @return void
	 */
	public static function ignorePorts()
	{

		if (Helper::postRequest('ajax_action') !== 'ignore_ports') {
			return;
		}

		$template = new Template();

		$action = Helper::postRequest('port_action');
		$port = (int)Helper::postRequest('port');

		$host = API::siteInfo();

		switch ($action) {
			case 'add':
				$response = API::addIgnorePort($host['id'], $port);
				break;

			case 'remove':
				$response = API::removeIgnorePort($host['id'], $port);
				break;
		}

		if (!isset($response['errors'])) {

			$ports = API::getAllPortsList($host['id']);
			$open_ports[] = [
				'variables' => [
					"ports" => $ports,
				],
				'template' => 'open_ports',
			];

			$ignore_ports[] = [
				'variables' => [
					"ports" => $ports,
				],
				'template' => 'ignore_ports',
			];
			$response = [
				'open_ports' => $template->arrayRender($open_ports),
				'ignore_ports' => $template->arrayRender($ignore_ports),
			];

		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();

		echo json_encode($response);
	}

	/**
	 * Request for a report link.
	 *
	 * @return void
	 */
	public static function reports()
	{

		if (Helper::postRequest('ajax_action') !== 'reports') {
			return;
		}

		$template = new Template();

		$action = Helper::postRequest('report_action');

		switch ($action) {
			case 'download':
				$id = Helper::postRequest('id');
				$link = API::downloadReport($id);
				if ($link) {
					$response['link'] = $link;
				}
				break;
			case 'report_form':

				$period = explode(" to ", Helper::postRequest('date_period'));
				$modules_data = Helper::postRequest('modules');

				$modules = [
					'wa' => 'false',
					'dc' => 'false',
					'ps' => 'false',
					'rc' => 'false',
					'sc' => 'false',
					'av' => 'false',
					'waf' => 'false'
				];

				foreach ($modules_data as $module => $value) {
					$modules[$module] = 'true';
				}

				$host = API::siteInfo();
				$api_response = API::generateReport($host['id'], $period, $modules);

				if (!$api_response) {
					$massage = '<div class="message error_message">' . \JText::_('WTOTEM_REPORT_GENERATION_ERROR') . '</div>';
				} else {
					$data = API::getAllReports($host['id']);
					Cache::setData(['getAllReports' => $data], $host['id']);

					// Reports.
					$build[] = [
						'variables' => [
							"reports" => Helper::getReports($data['edges']),
							"has_next_page" => $data['pageInfo']['hasNextPage'],
						],
						'template' => 'reports_list',
					];

					// Reports mobile.
					$build_mobile[] = [
						'variables' => [
							"reports" => Helper::getReports($data['edges']),
							"has_next_page" => $data['pageInfo']['hasNextPage'],
						],
						'template' => 'reports_list_mobile',
					];

					$response = [
						'reports' => $template->arrayRender($build),
						'reports_m' => $template->arrayRender($build_mobile),
						'link' => $api_response,
					];

					$massage = '<div class="message success_message">' . \JText::_('WTOTEM_THE_REPORT_WAS_SUCCESSFULLY_GENERATED') . '</div>';
				}

				$response['massage'] = $massage;

				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();
		echo json_encode($response);
	}

	/**
	 * Request for a report link.
	 *
	 * @return void
	 */
	public static function settings()
	{

		if (Helper::postRequest('ajax_action') !== 'settings') {
			return;
		}

		$av_installed = Option::getOption('av_installed');
		$waf_installed = Option::getOption('waf_installed');
		$action = Helper::postRequest('settings_action');

		if (in_array($action, ['module_toggle', 'module_notifications', 'waf_settings', 'add_allow_ip', 'add_deny_ip', 'add_allow_url', 'add_ip_list', 'country_blocking'])) {
			if (!$av_installed && !$waf_installed) {
				Option::setNotification('warning', \JText::_('WTOTEM_NOT_POSSIBLE_MAKE_CHANGES'));

				echo json_encode([
					'success' => false,
					'notifications' => self::notifications()
				]);
				return;
			}
		}

		$host = API::siteInfo();
		$template = new Template();

		switch ($action) {

			case 'module_toggle':
				$config = API::toggleConfigs(Helper::postRequest('value'));

				$configs_data = API::getConfigs($host['id']);
				Cache::setData(['getConfigs' => $configs_data], $host['id']);

				Option::setNotification('success', \JText::_('WTOTEM_YOUR_CHANGES_APPLIED_SUCCESSFULLY'));
				$response['isActive'] = $config['isActive'];
				$response['success'] = true;
				break;

			case 'country_blocking':
				$countries = Helper::postRequest('checked_countries');

				if (API::syncBlockedCountries($host['id'], $countries)) {
					$waf_data = API::getBlockedCountries($host['id']);
					Cache::setData(['getBlockedCountries' => $waf_data], $host['id']);

					Option::setNotification('success', \JText::_('WTOTEM_YOUR_CHANGES_APPLIED_SUCCESSFULLY'));
					$response['blocked_countries_list'] = $waf_data['blockedCountries'];
					$response['success'] = true;
				} else {
					Option::setNotification('success', \JText::_('WTOTEM_YOUR_CHANGES_HAVE_NOT_BEEN_APPLIED'));
					$response['success'] = false;
				}

				break;

			case 'module_notifications':
				$config = API::toggleNotifications($host['id'], Helper::postRequest('value'));

				$configs_data = API::getConfigs($host['id']);
				Cache::setData(['getConfigs' => $configs_data], $host['id']);

				Option::setNotification('success', \JText::_('WTOTEM_YOUR_CHANGES_APPLIED_SUCCESSFULLY'));
				$response['isActive'] = $config;
				$response['success'] = true;
				break;

			case 'waf_settings':

				$response['success'] = true;

				$dos = Helper::postRequest('dos');
				$dos_limit = Helper::postRequest('dos_limit');
				$login_attempt = Helper::postRequest('login_attempt');
				$login_attempt_limit = Helper::postRequest('login_attempt_limit');

				if ($dos) {
					if (empty($dos_limit)) {
						$response['errors']['dos_limit'] = \JText::_('WTOTEM_THE_FIELD_IS_REQUIRED');
					} else if ($dos_limit < 500 or $dos_limit > 10000) {
						$response['success'] = false;
						$response['errors']['dos_limit'] = sprintf(\JText::_('WTOTEM_PLEASE_SPECIFY_A_VALUE_FROM_TO'), '500', '10 000');
					}
				}

				if ($login_attempt) {
					if (empty($login_attempt_limit)) {
						$response['errors']['login_attempt_limit'] = \JText::_('WTOTEM_THE_FIELD_IS_REQUIRED');
					} else if ($login_attempt_limit < 5 or $login_attempt_limit > 30) {
						$response['success'] = false;
						$response['errors']['login_attempt_limit'] = sprintf(\JText::_('WTOTEM_PLEASE_SPECIFY_A_VALUE_FROM_TO'), '5', '30');
					}
				}

				if (!$response['success']) {
					break;
				} else {
					$response['errors'] = false;
				}

				$settings = [
					'gdn' => Helper::postRequest('gdn'),
					'dosProtection' => $dos,
					'dosLimit' => $dos_limit,
					'loginAttemptsProtection' => $login_attempt,
					'loginAttemptsLimit' => $login_attempt_limit,
				];

				$host = API::siteInfo();
				$api_response = API::setFirewallSettings($host['id'], $settings);

				if (!$api_response['errors']) {

					$data = API::getIpLists($host['id']);
					Cache::setData(['getIpLists' => $data], $host['id']);

					Option::setNotification('success', \JText::_('WTOTEM_YOUR_CHANGES_APPLIED_SUCCESSFULLY'));
				}

				break;

			case 'add_allow_ip':
				$api_response = API::addIpToList($host['id'], Helper::postRequest('value'), 'white');
				if ($api_response) {
					$data = API::getIpLists($host['id']);
					Cache::setData(['getIpLists' => $data], $host['id']);
					$build[] = [
						'variables' => [
							"list" => Helper::getIpList($data['whiteList'], 'ip_allow'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
				}

				$response['success'] = true;
				break;

			case 'add_deny_ip':
				$api_response = API::addIpToList($host['id'], Helper::postRequest('value'), 'black');
				if ($api_response) {
					$data = API::getIpLists($host['id']);
					Cache::setData(['getIpLists' => $data], $host['id']);
					$build[] = [
						'variables' => [
							"list" => Helper::getIpList($data['blackList'], 'ip_deny'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
				}

				$response['success'] = true;
				break;

			case 'add_allow_url':
				$api_response = API::addUrlToAllowList($host['id'], Helper::postRequest('value'));
				if ($api_response) {
					$data = API::getAllowUrlList($host['id']);
					$build[] = [
						'variables' => [
							"list" => Helper::getUrlAllowList($data),
						],
						'template' => 'allow_url_list',
					];

					$response['content'] = $template->arrayRender($build);
				}

				$response['success'] = true;
				break;

			case 'add_ip_list':
				$ips = Helper::postRequest('ips');
				$list_name = Helper::postRequest('list');

				$host = API::siteInfo();
				$api_response = API::addIpToList($host['id'], $ips, $list_name);

				if ($api_response) {
					$data = API::getIpLists($host['id']);

					$data_list = ($list_name == 'white') ? $data['whiteList'] : $data['blackList'];
					$ip_list = ($list_name == 'white') ? 'ip_allow' : 'ip_deny';

					$build[] = [
						'variables' => [
							"list" => Helper::getIpList($data_list, $ip_list),
						],
						'template' => 'allow_deny_list',
					];

					if ($api_response['status'] != 0) {
						$response['invalidIPs'] = implode("\n", $api_response['invalidIPs']);
					}

					$response['wrap'] = ($list_name == 'white') ? '#wtotem_ip_allow_list' : '#wtotem_ip_deny_list';
					$response['content'] = $template->arrayRender($build);
				}
				$response['success'] = true;

				break;
		}

		$response['notifications'] = self::notifications();
		echo json_encode($response);
	}

	/**
	 * Request to remove from the list of deny/allowed ip or url addresses.
	 *
	 * @return void
	 */
	public static function remove()
	{

		if (Helper::postRequest('ajax_action') !== 'remove') {
			return;
		}

		$av_installed = Option::getOption('av_installed');
		$waf_installed = Option::getOption('waf_installed');

		if (!$av_installed && !$waf_installed) {
			Option::setNotification('warning', \JText::_('WTOTEM_NOT_POSSIBLE_MAKE_CHANGES'));

			echo json_encode([
				'success' => false,
				'notifications' => self::notifications()
			]);
			return;
		}

		$action = Helper::postRequest('remove_action');
		$host = API::siteInfo();
		$template = new Template();

		switch ($action) {
			case 'ip_allow':
				$api_response = API::removeIpFromList(Helper::postRequest('id'));

				if ($api_response) {
					$data = API::getIpLists($host['id']);

					$build[] = [
						'variables' => [
							"list" => Helper::getIpList($data['whiteList'], 'ip_allow'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
					$response['wrap'] = '#wtotem_ip_allow_list';
				}
				break;

			case 'ip_deny':
				$api_response = API::removeIpFromList(Helper::postRequest('id'));

				if ($api_response) {
					$data = API::getIpLists($host['id']);

					$build[] = [
						'variables' => [
							"list" => Helper::getIpList($data['blackList'], 'ip_deny'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
					$response['wrap'] = '#wtotem_ip_deny_list';
				}
				break;

			case 'url_allow':
				$api_response = API::removeUrlFromAllowList(Helper::postRequest('id'));

				if ($api_response) {
					$data = API::getAllowUrlList($host['id']);

					$build[] = [
						'variables' => [
							"list" => Helper::getUrlAllowList($data),
						],
						'template' => 'allow_url_list',
					];

					$response['content'] = $template->arrayRender($build);
					$response['wrap'] = '#wtotem_allow_url';
				}
				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();
		echo json_encode($response);
	}

	/**
	 * Changing the theme mode.
	 *
	 * @return void
	 */
	public static function changeThemeMode()
	{

		if (Helper::postRequest('ajax_action') !== 'theme_mode') {
			return;
		}

		$theme_mode = Option::getSessionOption('theme_mode');

		if ($theme_mode == 'dark') {
			Option::setSessionOptions(['theme_mode' => 'light']);
			$response = 'light';
		} else {
			Option::setSessionOptions(['theme_mode' => 'dark']);
			$response = 'dark';
		}

		echo json_encode($response);
	}

	/**
	 * Set user time zone offset.
	 *
	 * @return void
	 */
	public static function userTimeZone()
	{

		if (Helper::postRequest('ajax_action') !== 'set_time_zone') {
			return;
		}

		$time_zone_offset = Helper::postRequest('offset');
		$now = strtotime('now');
		$check = Option::getOption('time_zone_check') ?: 0;

		// Checking whether an hour has elapsed since the previous request.
		if ($now >= $check) {
			$time_zone = API::getTimeZone();
			if ($time_zone) {
				$time_zone_offset = timezone_offset_get(new \DateTimeZone($time_zone), new \DateTime('now', new \DateTimeZone('Europe/London'))) / 3600;
				Option::setOptions(['time_zone_check' => $now + 3600]);
			}
			Option::setOptions(['time_zone_offset' => $time_zone_offset]);
		}

		echo json_encode([
			'success' => true,
			'time_zone_offset' => $time_zone_offset
		]);

	}

	/**
	 * Notification output.
	 *
	 * @return string
	 */
	public static function notifications()
	{

		$notifications = Helper::getNotifications();

		if ($notifications) {
			$build[] = [
				'variables' => [
					'notifications' => $notifications,
				],

				'template' => 'notifications',
			];

			$template = new Template();
			return $template->arrayRender($build);
		}
		return false;

	}


}
