<?php
/**
 * @name		WebTotem Security
 * @package		pkg_wtotem
 * @copyright	Copyright (C) 2023. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 * @author		WebTotem team
 */


Namespace WebTotem;

// No direct access
defined('_JEXEC') or die;
defined('WEBTOTEM_INIT') or die;

use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

/**
 * WebTotem base helper class.
 */
class Helper {

	static protected $input;

	/**
	 * Create an object of the Input class.
	 *
	 * @return object
	 *    Object of the Input class.
	 */
	public static function getInput() {
		if (empty(self::$input)) {
			self::$input = Factory::getApplication()->input;
		}
		return self::$input;
	}

	/**
	 * Create an object of the Input class.
	 *
	 * @return object
	 *    Object of the Input class.
	 */
	public static function postRequest($name) {
		if (empty(self::$input)) {
			self::$input = Factory::getApplication()->input;
		}
		return self::$input->get($name, '');
	}

	/**
	 * Load CSS and JS.
	 */
	public static function load() {
		\JHtml::_('jquery.framework');
		$doc = \JFactory::getDocument();

		$input = Helper::getInput();
		$task = $input->get('task', '');

		$css_path = WTOTEM_MEDIA_URI . '/assets/css/';
		$doc->addStylesheet($css_path . 'flatpickr.min.css' . self::fileVersion('css/flatpickr.min.css'));
		$doc->addStylesheet($css_path . 'toastr.min.css' . self::fileVersion('css/toastr.min.css'));
		$doc->addStylesheet($css_path . 'main.css'. self::fileVersion('css/main.css'));

		$js_path = WTOTEM_MEDIA_URI . '/assets/js/';

		if($task == 'settings'){
			$doc->addScript($js_path . 'country-blocking.js'. self::fileVersion('js/country-blocking.js'), "text/javascript", true);
		}

		$doc->addScript($js_path . 'jsdelivr_chart.js'. self::fileVersion('js/jsdelivr_chart.js'), "text/javascript", true);
		$doc->addScript($js_path . 'flatpickr.js'. self::fileVersion('js/flatpickr.js'), "text/javascript", true);
		$doc->addScript($js_path . 'd3.v4.js'. self::fileVersion('js/d3.v4.js'), "text/javascript", true);
		$doc->addScript($js_path . 'progress_bar.js'. self::fileVersion('js/progress_bar.js'), "text/javascript", true);
		$doc->addScript($js_path . 'toastr.min.js'. self::fileVersion('js/toastr.min.js'), "text/javascript", true);
		$doc->addScript($js_path . 'chart.js'. self::fileVersion('js/chart.js'), "text/javascript", true);
		$doc->addScript($js_path . 'main.js'. self::fileVersion('js/main.js'), "text/javascript", true);

	}


	/**
	 * Returns the md5 hash representing the content of a file.
	 *
	 * @param  string $file
	 *    Relative path to the file.
	 * @return string
	 *    Seven first characters in the hash of the file.
	 */
	public static function fileVersion($file = '') {
		return '?ver=' . substr(md5_file(WTOTEM_MEDIA_PATH . '/assets/' . $file), 0, 7);
	}

	/**
	 * Returns full path to image.
	 *
	 * @param string $image
	 *    Relative path to the file.
	 * @return string
	 *    Full path to image.
	 */
	public static function getImagePath($image) {
		return WTOTEM_MEDIA_URI . '/images/' . $image;
	}

	/**
	 * Check that the training period has passed for the firewall.
	 *
	 * @param string $created_at
	 *   Date when the waf configuration was created.
	 *
	 * @return bool
	 *   Returns boolean.
	 */
	public static function isWafTraining($created_at) {
		if ($created_at) {
			$when_waf_trained = strtotime('+2 day', strtotime($created_at));
			$today = strtotime('today');

			return ($when_waf_trained < $today) ? FALSE : TRUE;
		}
		return FALSE;
	}

	/**
	 * Check if the data for the period is available.
	 *
	 * @param string $created_at
	 *   Date when the agent manager was created.
	 *
	 * @return array
	 *   Returns an array with periods.
	 */
	public static function isPeriodAvailable($created_at) {

		$diff = strtotime(gmdate("Y-m-d H:i:s")) - strtotime($created_at);
		$daysCount = floor($diff / 86400);

		return [
				'monthly' => $daysCount > 7,
				'yearly' => $daysCount > 30,
		];

	}

	/**
	 * Converting a date to the appropriate format.
	 *
	 * @param string $date
	 *   Date in any format.
	 * @param string $format
	 *   The format to which you want to convert the date.
	 *
	 * @return string
	 *   Returns converted Date.
	 */
	public static function dateFormatter($date, $format = 'M j, Y \/ H:i') {
		if (!$date) {
			return \JText::_('WTOTEM_UNKNOWN');
		}

		$time_zone = Option::getOption('time_zone_offset');
		$user_time = ($time_zone) ? strtotime($time_zone . 'hours', strtotime($date)) : strtotime($date);

		return HtmlHelper::date($user_time, Text::_($format));;
	}

	/**
	 * Get theme mode data.
	 *
	 * @return array
	 *   Returns array with current theme data.
	 */
	public static function getThemeMode() {
		$theme_mode = Option::getSessionOption('theme_mode');
		return [
				"is_dark_mode" => $theme_mode == 'dark' ? 'wtotem_theme—dark' : '',
				"dark_mode_checked" => $theme_mode == 'dark' ? 'checked' : '',
		];
	}

	/**
	 * Get current user language.
	 *
	 * @return string
	 *   Returns current language in 2-letter abbreviations
	 */
	public static function getLanguage() {
		$lang = \JFactory::getLanguage();
		$current_language = substr( $lang->getTag(), 0, 2);
		$language = (in_array($current_language, ['ru', 'en', 'pl'])) ? $current_language : 'en';
		return $language;
	}

	/**
	 * Converting a date to the appropriate format.
	 *
	 * @param string|array $days
	 *   Number of days or period to convert.
	 *
	 * @return array
	 *   Returns an array of two values "from" and "to"
	 */
	public static function getPeriod($days) {

		if (!$days) {
			$days = 30;
		}

		switch ($days) {

			case is_array($days):
				$to = $days[1] ?: $days[0];
				$period = [
						'from' => strtotime(date('Y-m-d 00:00:01', strtotime($days[0]))),
						'to' => strtotime(date('Y-m-d 23:59:59', strtotime($to))),
				];
				break;

			case $days <= 1:
				$period = [
						'from' => strtotime('-24 hours'),
						'to' => time(),
				];
				break;

			default:
				$period = [
						'from' => time() - ($days * 86400),
						'to' => time(),
				];
		}

		return $period;
	}

	/**
	 * Convert an array to a string with quotation marks.
	 *
	 * @param array $array
	 *   Data array.
	 *
	 * @return string
	 *   Array of data converted to string.
	 */
	public static function convertArrayToString($array) {
		if (empty($array)) {
			return '';
		}
		return '"' . implode('","', $array) . '"';
	}

	/**
	 * Converting the response to a readable form.
	 *
	 * @param string $message
	 *   Message response from the API server to the request.
	 *
	 * @return string|bool
	 *   Returns a message.
	 */
	public static function messageForHuman($message) {

		$definition = $message;

		switch ($message) {
			case 'HOSTS_LIMIT_EXCEEDED':
				$definition = \JText::_('WTOTEM_LIMIT_OF_ADDING_SITES_EXCEEDED');
				break;

			case 'USER_ALREADY_REGISTERED':
				$definition = \JText::_('WTOTEM_USER_ALREADY_EXISTS');
				break;

			case 'DUPLICATE_HOST':
				$definition = \JText::_('WTOTEM_DUPLICATE_HOST');
				break;

			case 'INVALID_DOMAIN_NAME':
				$definition = \JText::_('WTOTEM_INVALID_DOMAIN_NAME');
				break;
			default:
				$definition = str_replace("_", " ", $definition);
				$definition = ucfirst(strtolower($definition));

		}
		return $definition;
	}

	/**
	 * Get the data associated with the status.
	 *
	 * @param string $status
	 *   Module or agent status.
	 *
	 * @return array
	 *   Returns an array with status data.
	 */
	public static function getStatusData($status) {
		$path = self::getImagePath('');
		$status = ($status == "installed") ? 'working' : $status;

		switch ($status) {

			case 'clean':
			case 'up':
			case 'installed':
			case 'working':
				$status_data = [
						'class' => 'is--status--ok',
						'image' => $path . 'check-mark.svg',
						'icon' => $path . 'icon_success_status.svg',
				];
				break;

			case 'pending':
				$status_data = [
						'class' => 'is--status--pending',
						'image' => $path . 'loading.svg',
						'icon' => $path . 'alert-warning.svg',
				];
				break;

			case 'pause':
			case 'modified':
				$status_data = [
						'class' => 'is--status--pending',
						'image' => $path . 'warning.svg',
						'icon' => $path . 'alert-warning.svg',
				];
				break;

			case 'expired':
			case 'no_cert':
			case 'expires':
			case 'open_ports':
			case 'not_supported':
			case 'not_registered':
				$status_data = [
						'class' => 'is--status--warning',
						'image' => $path . 'warning.svg',
						'icon' => $path . 'alert-warning.svg',
				];
				break;

			case 'invalid':
			case 'error':
			case 'down':
			case 'expires_today':
			case 'infected':
			case 'deface':
			case 'not_installed':
				$status_data = [
						'class' => 'is--status--error',
						'image' => $path . 'warning.svg',
						'icon' => $path . 'alert-warning.svg',
				];
				break;

			default:
				$status_data = [
						'class' => 'is--status--pending',
						'image' => $path . 'warning.svg',
						'icon' => $path . 'alert-warning.svg',
				];
		}
		$status_data['name'] = $status;
		$status_data['text'] = self::getStatusText($status);
		$status_data['tooltips'] = self::getTooltips($status);

		return $status_data;
	}

	/**
	 * Get a readable status text.
	 *
	 * @param string $status
	 *   Module or agent status.
	 *
	 * @return string
	 *   Returns the status text in the current language.
	 */
	public static function getStatusText($status) {
		$statuses = [
				'warning' => \JText::_('WTOTEM_WARNING'),
				'error' => \JText::_('WTOTEM_ERROR'),
				'success' => \JText::_('WTOTEM_SUCCESS'),
				'info' => \JText::_('WTOTEM_INFO'),
				'invalid' => \JText::_('WTOTEM_INVALID'),
				'ok' => \JText::_('WTOTEM_EVERYTHING_IS_OK'),
				'expired' => \JText::_('WTOTEM_EXPIRED'),
				'expires' => \JText::_('WTOTEM_EXPIRES'),
				'expires_today' => \JText::_('WTOTEM_EXPIRES_TODAY'),
				'missing' => \JText::_('WTOTEM_MISSING'),
				'active' => \JText::_('WTOTEM_ACTIVE'),
				'inactive' => \JText::_('WTOTEM_INACTIVE'),
				'pending' => \JText::_('WTOTEM_PENDING'),
				'pause' => \JText::_('WTOTEM_DISABLED'),
				'available' => \JText::_('WTOTEM_AVAILABLE'),
				'not_supported' => \JText::_('WTOTEM_NOT_SUPPORTED'),
				'not_registered' => \JText::_('WTOTEM_NOT_REGISTERED'),
				'unsupported' => \JText::_('WTOTEM_UNSUPPORTED'),
				'clean' => \JText::_('WTOTEM_CLEAN'),
				'clear' => \JText::_('WTOTEM_CLEAR'),
				'blacklisted' => \JText::_('WTOTEM_INFECTED'),
				'miner_detected' => \JText::_('WTOTEM_INFECTED'),
				'deface' => \JText::_('WTOTEM_DEFACE'),
				'modified' => \JText::_('WTOTEM_MODIFIED'),
				'detected' => \JText::_('WTOTEM_DETECTED'),
				'open_ports' => \JText::_('WTOTEM_OPEN_PORTS'),
				'blocked' => \JText::_('WTOTEM_BLOCKED'),
				'connected' => \JText::_('WTOTEM_CONNECTED'),
				'attacks_detected' => \JText::_('WTOTEM_ATTACKS_DETECTED'),
				'signature_found' => \JText::_('WTOTEM_SIGNATURE_FOUND'),
				'file_changes' => \JText::_('WTOTEM_FILE_CHANGES'),
				'no_cert' => \JText::_('WTOTEM_NO_CERT'),
				'down' => \JText::_('WTOTEM_DOWN'),
				'up' => \JText::_('WTOTEM_UP'),
				'infected' => \JText::_('WTOTEM_INFECTED'),
				'not_installed' => \JText::_('WTOTEM_NEED_TO_INSTALL'),
				'agent_not_available' => \JText::_('WTOTEM_AGENT_NOT_AVAILABLE'),
				'update_error' => \JText::_('WTOTEM_UPDATE_ERROR'),
				'session_error' => \JText::_('WTOTEM_SESSION_ERROR'),
				'internal_error' => \JText::_('WTOTEM_INTERNAL_ERROR'),
				'installing' => \JText::_('WTOTEM_INSTALLING'),
				'installed' => \JText::_('WTOTEM_INSTALLED'),
				'working' => \JText::_('WTOTEM_WORKING'),
				"critical" => \JText::_('WTOTEM_CRITICAL'),
				"deleted" => \JText::_('WTOTEM_DELETED'),
				"changed" => \JText::_('WTOTEM_CHANGED'),
				"new" => \JText::_('WTOTEM_NEW'),
				"scanned" => \JText::_('WTOTEM_SCANNED'),
				"quarantine" => \JText::_('WTOTEM_IN_QUARANTINE'),
		];

		return (array_key_exists($status, $statuses)) ? $statuses[$status] : $status;
	}

	/**
	 * Get tooltips text for status.
	 *
	 * @param string $status
	 *   Module or agent status.
	 *
	 * @return string
	 *   Returns the status tooltip in the current language.
	 */
	public static function getTooltips($status) {
		$tooltips = [
				'invalid' => \JText::_('WTOTEM_INVALID_TOOLTIPS'),
				'expired' => \JText::_('WTOTEM_EXPIRED_TOOLTIPS'),
				'expires' => \JText::_('WTOTEM_EXPIRES_TOOLTIPS'),
				'expires_today' => \JText::_('WTOTEM_EXPIRES_TODAY_TOOLTIPS'),
				'error' => \JText::_('WTOTEM_ERROR_TOOLTIPS'),
				'pending' => \JText::_('WTOTEM_PENDING_TOOLTIPS'),
				'pause' => \JText::_('WTOTEM_PAUSE_TOOLTIPS'),
				'clean' => \JText::_('WTOTEM_CLEAN_TOOLTIPS'),
				'deface' => \JText::_('WTOTEM_DEFACE_TOOLTIPS'),
				'open_ports' => \JText::_('WTOTEM_OPEN_PORTS_TOOLTIPS'),
				'blocked' => \JText::_('WTOTEM_BLOCKED_TOOLTIPS'),
				'no_cert' => \JText::_('WTOTEM_NO_CERT_TOOLTIPS'),
				'down' => \JText::_('WTOTEM_DOWN_TOOLTIPS'),
				'up' => \JText::_('WTOTEM_UP_TOOLTIPS'),
				'infected' => \JText::_('WTOTEM_INFECTED_TOOLTIPS'),
				'installing' => \JText::_('WTOTEM_INSTALLING_TOOLTIPS'),
				'agent_not_available' => \JText::_('WTOTEM_AGENT_NOT_AVAILABLE_TOOLTIPS'),
				'update_error' => \JText::_('WTOTEM_UPDATE_ERROR_TOOLTIPS'),
				'session_error' => \JText::_('WTOTEM_SESSION_ERROR_TOOLTIPS'),
				'internal_error' => \JText::_('WTOTEM_INTERNAL_ERROR_TOOLTIPS'),
				'working' => \JText::_('WTOTEM_WORKING_TOOLTIPS'),
				'installed' => \JText::_('WTOTEM_INSTALLED_TOOLTIPS'),
				'not_installed' => \JText::_('WTOTEM_NOT_INSTALLED_TOOLTIPS'),
		];

		return (array_key_exists($status, $tooltips)) ? $tooltips[$status] : '';
	}

	/**
	 * Get reports with modules list.
	 *
	 * @param array $edges
	 *   Data on generated reports.
	 *
	 * @return array
	 *   Returns an array with converted data.
	 */
	public static function getReports(array $edges) {
		$modulesLang = [
				'wa' => \JText::_('WTOTEM_AVAILABILITY_LOG'),
				'dc' => \JText::_('WTOTEM_DEFACE_LOG'),
				'ps' => \JText::_('WTOTEM_PORT_LOG'),
				'rc' => \JText::_('WTOTEM_REPUTATION_LOG'),
				'sc' => \JText::_('WTOTEM_SCORING_LOG'),
				'av' => \JText::_('WTOTEM_ANTIVIRUS_LOG'),
				'waf' => \JText::_('WTOTEM_FIREWALL_LOG'),
		];

		$reports = [];

		foreach ($edges as $edge) {
			if (in_array(FALSE, $edge["node"])) {
				$arr = [];
				foreach ($edge["node"] as $module => $value) {
					if ($value == TRUE && array_key_exists($module, $modulesLang)) {
						$arr[] = $modulesLang[$module];
					}
				}
				$modules = implode(", ", $arr);
			} else {
				$modules = \JText::_('WTOTEM_ALL_MODULES');
			}

			$reports[] = [
					'id' => $edge["node"]['id'],
					'modules' => $modules,
					'created_at' => self::dateFormatter($edge["node"]['createdAt']),
			];
		}

		return $reports;
	}

	/**
	 * Get reputation status description.
	 *
	 * @param string $status
	 *   Reputation status.
	 *
	 * @return string
	 *   Returns a description of the reputation status
	 */
	public static function getReputationInfo($status) {
		switch ($status) {
			case 'clean':
				$data = \JText::_('WTOTEM_YOUR_REPUTATION_IS_GOOD');
				break;

			case 'infected':
				$data = \JText::_('WTOTEM_OH_YOUR_REPUTATION_IS_BAD');
				break;

			default:
				$data = \JText::_('WTOTEM_INFORMATION_IS_BEING_UPDATED');
		}
		return $data;
	}

	/**
	 * Get blacklists entries counts.
	 *
	 * @param string $status
	 *   Reputation status.
	 * @param array $virus_list
	 *   Sources where the site can be blacklisted.
	 *
	 * @return int
	 *   Number of references in blacklists.
	 */
	public static function blacklistsEntries($status, array $virus_list) {
		$count = 0;
		if ($status != "clean") {
			foreach ($virus_list as &$list) {
				if (!empty($list['virus']['type'])) {
					$count++;
				}
			}
		}
		return $count;
	}

	/**
	 * Classification of the rating in the letter grades.
	 *
	 * @param int $score
	 *   Site rating from 1 to 100.
	 *
	 * @return array
	 *   Returns an array of data.
	 */
	public static function scoreGrading($score) {
		if ($score < 0 || $score > 100) {
			return ['grade' => '', 'color' => ''];
		}

		$scores = [
				100 => 'A+',
				90 => 'A',
				80 => 'A-',
				70 => 'B+',
				60 => 'B',
				50 => 'B-',
				35 => 'C+',
				20 => 'C',
				0 => 'C-',
		];

		foreach ($scores as $key => $value) {
			if ($score >= $key) {
				$grade = $value;
				break;
			}
		}

		// Set a color depending on the grade.
		switch ($score) {
			case $score >= 80:
				$color = 'green';
				break;

			case $score >= 50:
				$color = 'orange';
				break;

			default:
				$color = 'red';
		}

		return ['grade' => $grade, 'color' => $color];
	}

	/**
	 * Calculate the number of remaining days.
	 *
	 * @param string $date
	 *   Expiry date.
	 *
	 * @return string
	 *   Returns the number of days before the expiration date.
	 */
	public static function daysLeft($date) {
		if ((int)$date === 0) {
			$days_left = 0;
		} else {
			$now = new \DateTime();
			$expiry_date = new \DateTime();
			$timestamp = strtotime($date);
			$expiry_date->setTimestamp($timestamp);
			$days_left = $expiry_date->diff($now)->format("%a");
		}
		return $days_left;
	}

	/**
	 * Converting the firewall logs.
	 *
	 * @param array $logs_
	 *   Firewall logs from WebTotem.
	 *
	 * @return array
	 *   Converted array of logs.
	 */
	public static function wafLogs(array $logs_) {
		$logs = [];
		foreach ($logs_ as $key => $log) {
			$log = $log['node'];

			$request = urldecode($log['request']);
			$payload = urldecode($log['payload']);

			$logs[$key]['ip'] = $log['ip'];
			$logs[$key]['request'] = htmlspecialchars($request);
			$logs[$key]['time'] = self::dateFormatter($log['time']);
			$logs[$key]['country_code'] = strtolower($log['country']);
			$logs[$key]['country'] = $log['location']['country']['nameEn'];
			$logs[$key]['blocked'] = $log['blocked'] ? \JText::_('WTOTEM_BLOCKED_IP') : \JText::_('WTOTEM_NOT_BLOCKED');

			$more = [
					'ip' => $log['ip'],
					'proxy_ip' => $log['proxyIp'],
					'source' => $log['source'],
					'request' => htmlspecialchars($request),
					'user_agent' => $log['userAgent'],
					'time' => self::dateFormatter($log['time']),
					'type' => $log['type'],
					'category' => $log['category'],
					'country' => $log['location']['country']['nameEn'],
					'payload' => htmlspecialchars($payload),
			];

			$logs[$key]['more'] = json_encode($more);
		}
		return $logs;
	}

	/**
	 * Converting firewall data to json for a D3 chart.
	 *
	 * @param array $charts
	 *   Charts data from WebTotem.
	 *
	 * @return array
	 *   Returns the converted data for chart.
	 */
	public static function generateWafChart(array $charts) {
		$sum = 0;
		foreach ($charts as $chart) {
			$sum += $chart['attacks'];
		}
		if ($sum == 0) {
			return ['chart' => FALSE, 'count_attacks' => 0, 'count_blocks' => 0];
		}

		// Get days count.
		$charts_ = $charts;
		$first = array_shift($charts_);
		$last = array_pop($charts_);
		$days = ceil((strtotime($last['time']) - strtotime($first['time'])) / 86400);

		// Set variables.
		$count_attacks = $count_blocks = 0;

		foreach ($charts as $chart) {
			if ($days <= 1) {
				$time_zone = Option::getOption('time_zone_offset');
				$userTime = ($time_zone) ? strtotime($time_zone . ' hours', strtotime($chart['time'])) : strtotime($chart['time']);
			}
			if (($chart['attacks'] and $days == 2) or $days != 2) {
				$result[] = [
						'date' => ($days <= 1) ? date("Y-m-d H:00:00", $userTime) : date("Y-m-d", strtotime($chart['time'])),
						'count' => $chart['blocked'],
						'attacks' => $chart['attacks'],
						'blocked' => $chart['blocked'],
				];
				$count_attacks += $chart['attacks'];
				$count_blocks += $chart['blocked'];
			}
		}

		if (!isset($result)) {
			return [
					'chart' => FALSE,
					'count_attacks' => 0,
					'count_blocks' => 0,
					'days' => 0,
			];
		}

		return [
				'chart' => json_encode($result),
				'count_attacks' => $count_attacks,
				'count_blocks' => $count_blocks,
				'days' => $days,
		];
	}

	/**
	 * Converting data to json for a D3 chart.
	 *
	 * @param array $charts
	 *   Charts data from WebTotem.
	 * @param int $days
	 *   The number of days to build the chart.
	 *
	 * @return bool|string
	 *   Returns the converted data for chart.
	 */
	public static function generateChart(array $charts, int $days = 7) {
		$sum = 0;
		foreach ($charts as $chart) {
			$sum += $chart['value'];
		}
		if ($sum == 0) {
			return FALSE;
		}

		$result = [];

		foreach ($charts as $chart) {
			if ($days <= 1) {
				$time_zone = Option::getOption('time_zone_offset');
				$userTime = ($time_zone) ? strtotime($time_zone . 'hours', strtotime($chart['time'])) : strtotime($chart['time']);
			}
			$result[] = [
					'date' => ($days <= 1) ? date("Y-m-d H:00:00", $userTime) : date("Y-m-d", strtotime($chart['time'])),
					'value' => $chart['value'],
			];
		}

		return json_encode($result, TRUE);
	}

	/**
	 * Converting data to json for a D3 chart.
	 *
	 * @param array $data
	 *   Charts data from WebTotem.
	 *
	 * @return array|bool
	 *   Returns the converted data for chart.
	 */
	public static function generateAttacksMapChart(array $data) {
		$attacks = [];
		$countries = [];
		$labels = [];
		foreach ($data as $value) {
			$attacks[] = $value['attacks'];
			$labels[] = self::getCountryName($value['country']);
			$countries[] = $value['location']['country']['nameEn'];
		}
		$result = ['attacks' => $attacks, 'countries' => $countries, 'labels' => $labels];

		if (!$attacks) {
			return FALSE;
		}

		return json_encode($result, TRUE);
	}

	/**
	 * Reassembling the antivirus logs.
	 *
	 * @param array $logs_
	 *   Antivirus logs from WebTotem.
	 *
	 * @return array
	 *   Reassembled array of logs.
	 */
	public static function getAntivirusLogs(array $logs_) {
		$logs = [];
		foreach ($logs_ as $key => $log) {
			$log = $log['node'];

			$file_info = new SplFileInfo(urldecode($log['filePath']));

			$log['original_path'] = $log['filePath'];
			$log['file_path'] = $file_info->getPath() . '/';
			$log['file_name'] = $file_info->getFilename();
			$log['time'] = self::dateFormatter($log['time']);
			$log['permissions_changed'] = $log['permissionsChanged'];
			$log['status'] = self::getStatusData($log['event']);
			$log['class'] = 'wt-text--green';

			switch ($log['event']) {
				case 'modified':
				case 'quarantine':
					$log['class'] = "wt-text--yellow";
					break;

				case 'deleted':
					$log['class'] = "wt-text--light-gray";
					break;

				case 'infected':
					$log['class'] = "wt-text--red";
					break;
			}

			$logs[$key] = $log;
		}
		return $logs;
	}

	/**
	 * Reassembling the quarantine logs.
	 *
	 * @param array $logs_
	 *   Quarantine logs from WebTotem.
	 *
	 * @return array
	 *   Reassembled array of logs.
	 */
	public static function getQuarantineLogs(array $logs_) {
		$logs = [];
		foreach ($logs_ as $key => $log) {
			$logs[$key] = $log;
			$logs[$key]['path'] = urldecode($log['path']);
			$logs[$key]['date'] = self::dateFormatter($log['date']);
		}

		return $logs;
	}

	/**
	 * Generate an array of IP address data.
	 *
	 * @param array $data
	 *   IP addresses data from WebTotem.
	 * @param string $list_name
	 *   Allow or deny list.
	 *
	 * @return array
	 *   Returns array of data.
	 */
	public static function getIpList(array $data, $list_name) {
		$list = [];
		foreach ($data as $item) {
			$list[] = [
					'ip' => $item['ip'],
					'id' => $item['id'],
					'created_at' => self::dateFormatter($item['createdAt']),
					'list_name' => $list_name,
			];
		}
		return $list;
	}

	/**
	 * Generate an array of URL address data.
	 *
	 * @param array $data
	 *   URL addresses data from WebTotem.
	 *
	 * @return array
	 *   Returns array of data.
	 */
	public static function getUrlAllowList(array $data) {
		$list = [];
		foreach ($data as $item) {
			$list[] = [
					'url' => $item['url'],
					'id' => $item['id'],
					'created_at' => self::dateFormatter($item['createdAt']),
					'list_name' => 'url_allow',
			];
		}
		return $list;
	}

	/**
	 * Convert IP list to be transferred to WebTotem.
	 *
	 * @param string $data
	 *   IP list.
	 *
	 * @return string
	 *   Returns the converted string.
	 */
	public static function convertIpListForApi($data) {
		if (!$data) {
			return FALSE;
		}

		$ips = preg_split("/(?(?=[\s,])[^.]|^$)/", $data);

		if (is_array($ips)) {
			$ips_ = '[';
			foreach ($ips as $ip) {
				if (!empty($ip)) {
					$ips_ .= '"' . $ip . '",';
				}
			}
			$ips_ = substr($ips_, 0, -1);
			$ips_ .= ']';
		} else {
			$ips_ = '"' . $ips . '"';
		}

		return $ips_;
	}

	/**
	 * Get data of the country with the most attacks.
	 *
	 * @param array $map
	 *   Map logs from WebTotem.
	 *
	 * @return array
	 *   Returns array of data.
	 */
	public static function getMostAttacksData($map) {

		if ($map) {
			$most_attacks_key = array_search(max(array_column($map, 'attacks')), array_column($map, 'attacks'));
			$total_attacks = array_sum(array_column($map, 'attacks'));

			$data['percent'] = ($total_attacks) ? round($map[$most_attacks_key]['attacks'] / $total_attacks * 100) : 0;
			$data['country'] = self::getCountryName($map[$most_attacks_key]['country']);
			$data['offset'] = 176 / 100 * (100 - $data['percent']);

			return $data;
		}

		return ['percent' => 0, 'country' => FALSE, 'offset' => 0];
	}

	/**
	 * Get data on the three most attacking countries.
	 *
	 * @param array $map
	 *   Map logs from WebTotem.
	 *
	 * @return array
	 *   Returns array of data.
	 */
	public static function getTreeMostAttacksData($map) {
		$total_attacks = array_sum(array_column($map, 'attacks'));

		if ($map) {
			array_multisort(array_column($map, 'attacks'), SORT_DESC, $map);
			$data = array_slice($map, 0, 3);

			foreach ($data as $key => $value) {
				$data[$key]['percent'] = round($value['attacks'] / $total_attacks * 100);
				$data[$key]['country'] = self::getCountryName($value['country']);
			}

			return $data;
		}

		return [];
	}

	/**
	 * Getting the country name by two-letter code.
	 *
	 * @param string $key
	 *   Two-letter code.
	 *
	 * @return string
	 *   Returns country name.
	 */
	public static function getCountryName($key) {
		$countries = CountryManager::getStandardList();
		$key = (string)$key;

		return (array_key_exists($key, $countries)) ? $countries[$key] : $key;
	}

	/**
	 * Get configs data.
	 *
	 * @param array $array
	 *   Original array.
	 * @param string $key
	 *   The key to use as an index.
	 *
	 * @return array
	 *   Configs data array.
	 */
	public static function getConfigsData(array $array, $key) {
		$configs = self::arrayMapIndex($array, $key);

		foreach ($configs as $service => $config) {
			$configs[$service]['checked'] = ($config['isActive']) ? 'checked' : '';
			$configs[$service]['notification_checked'] = (isset($config['notifications']) && $config['notifications']) ? 'checked' : '';
		}

		return $configs;
	}

	/**
	 * Get waf setting data.
	 *
	 * @param array $settings
	 *   Original array.
	 *
	 * @return array
	 *   Configs data array.
	 */
	public static function getWafSettingData(array $settings) {
		$_settings['gdn']['checked'] = (isset($settings['gdn']) && !$settings['gdn']) ? '' : 'checked';
		$_settings['dos'] = [
				'checked' => (isset($settings['dosProtection']) && !$settings['dosProtection']) ? '' : 'checked',
				'visually' => (isset($settings['dosProtection']) && !$settings['dosProtection']) ? 'visually-hidden' : '',
		];
		$_settings['dos_limit'] = $settings['dosLimit'] ?: 1000;

		$_settings['login_attempt'] = [
				'checked' => (isset($settings['loginAttemptsProtection']) && !$settings['loginAttemptsProtection']) ? '' : 'checked',
				'visually' => (isset($settings['loginAttemptsProtection']) && !$settings['loginAttemptsProtection']) ? 'visually-hidden' : '',
		];
		$_settings['login_attempt_limit'] = $settings['loginAttemptsLimit'] ?: 20;

		return $_settings;
	}

	/**
	 * Replace array indexes by key.
	 *
	 * @param array $array
	 *   Original array.
	 * @param string $key
	 *   The key to use as an index.
	 *
	 * @return array
	 *   Returns a new array.
	 */
	public static function arrayMapIndex(array $array, $key) {
		$new_array = [];
		foreach ($array as $item) {
			if (array_key_exists($key, $item)) {
				$new_array[$item[$key]] = $item;
			}
		}
		return $new_array;
	}

	/**
	 * Generate random string.
	 *
	 * @param int $length
	 *   The required length of the string.
	 *
	 * @return string
	 *   Returns random string.
	 */
	public static function generateRandomString(int $length = 10): string
	{
		$characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_-';
		$charactersLength = strlen($characters);
		$randomString = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[rand(0, $charactersLength - 1)];
		}
		return $randomString;
	}

	/**
	 * Throw generic exception.
	 *
	 * @param string $message
	 *   Error or information message.
	 * @param string $type
	 *   Either info or error.
	 *
	 * @return bool
	 *    False all the time, used for debug.
	 * @throws Exception
	 *
	 */
	public static function throwException($message, $type = 'error') {
		if (defined('WTOTEM_THROW_EXCEPTIONS') && WTOTEM_THROW_EXCEPTIONS === true && is_string($message)) {
			$message = str_replace('<strong>WebTotem:</strong>', ($type === 'error' ? \JText::_('WTOTEM_ERROR') . ': ' : \JText::_('WTOTEM_INFO') . ': '), $message);
			throw new Exception($message, $type === 'error' ? 157 : 333);
		}
		return false;
	}

	/**
	 * Get notifications array.
	 *
	 * @return array
	 *   Returns notifications array.
	 */
	public static function getNotifications() {

		$notifications_data = Option::getNotificationsData();
		$notifications = [];

		foreach ($notifications_data as $notification) {
			switch ($notification['type']) {
				case 'error':
					$image = 'alert-error.svg';
					$class = 'wtotem_alert__title_red';
					break;

				case 'warning':
					$image = 'alert-warning.svg';
					$class = 'wtotem_alert__title_yellow';
					break;

				case 'success':
					$image = 'alert-success.svg';
					$class = 'wtotem_alert__title_green';
					break;

				case 'info':
					$image = 'info-blue.svg';
					$class = 'wtotem_alert__title_blue';
					break;
			}

			$notifications[] = [
					"text" => $notification['notice'],
					"id" => self::generateRandomString(8),
					"type" => self::getStatusText($notification['type']),
					"type_raw" => $notification['type'],
					"image" => $image,
					"class" => $class,
			];
		}

		return $notifications;
	}

	/**
	 * Get current agent installation statuses.
	 *
	 * @param array $agents_statuses
	 *   Agents statuses got from the WebTotem API.
	 *
	 * @return array
	 *   Returns an array with agent installation status data.
	 */
	public static function getAgentsStatuses(array $agents_statuses) {
		$agents = ['am', 'waf', 'av'];
		$installing_statuses = [
				'not_installed',
				'installing',
				'internal_error',
				'update_error',
				'config_error',
				'session_error',
		];

		$process_statuses = [];
		$option_statuses = [];

		foreach ($agents as $agent) {
			$status = AgentManager::checkInstalledService($agent);
			$option_statuses[$agent] = $status['option_status'] ?: FALSE;

			if ($agent == 'am') {
				if ($status['file_status']) {
					$process_statuses[$agent] = 'installed';
				} else {
					$process_statuses[$agent] = 'failed';
				}
			} else {
				if ($status['file_status']) {
					if (in_array($agents_statuses[$agent], $installing_statuses)) {
						$process_statuses[$agent] = 'installing';
					} elseif ($agents_statuses[$agent] == 'agent_not_available') {
						$process_statuses[$agent] = 'failed';
					} else {
						$process_statuses[$agent] = 'installed';
					}
				} else {
					$process_statuses[$agent] = 'installing';
				}
			}
		}

		return [
				'process_statuses' => $process_statuses,
				'option_statuses' => $option_statuses,
		];
	}

}
