<?php
/**
 * @name		WebTotem Security
 * @package		pkg_wtotem
 * @copyright	Copyright (C) 2023. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 * @author		WebTotem team
 */

Namespace WebTotem;

// No direct access
defined('_JEXEC') or die;
defined('WEBTOTEM_INIT') or die;

use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Factory;

class PageHandler {

	/**
	 * @throws \Exception
	 */
	public function execute() {

		$input = Helper::getInput();
		$task = $input->get('task', '');

		$app = \JFactory::getApplication();

		if($task != 'ajax'){

			$title = 'WebTotem';
			$icon  = 'wtotem';
			ToolbarHelper::title($title, $icon);

			// Execute pre-checks before every page
			if(!Option::isActivated() and $task !== 'activation') {
				// If the plugin is not activated by the API key, then redirect to the activation page.
				$app->redirect(\JRoute::_(WTOTEM_URL . '&task=activation'));
				exit;
			}
			elseif (Option::isActivated() and ($task === 'activation' or $task == '')){
				// If the plugin is activated by the API key, then redirect to the main page.
				$app->redirect(\JRoute::_(WTOTEM_URL . '&task=dashboard'));
				exit;
			}
			elseif(Option::isActivated()) {
				// Checking whether agents are installed, if they are not installed, then install.
				self::checkAgents();
			}
			// Check if the plugin version has changed.
			AgentManager::checkVersion();

		}

		// Load page and ajax handlers
		if (! $task) $task = '';
		if (is_callable(array($this, $task))) {
			return $this->$task();
		} else {
			throw new \Exception(\JText::sprintf('ERROR_TASK_NOT_FOUND', $task), 404);
		}

	}

	/**
	 * Checking whether agents are installed, if they are not installed, then install.
	 */
	private static function checkAgents(){

		$api_key = Option::getOption('api_key');

		$host = API::siteInfo();

		if ($api_key && array_key_exists('id', $host)) {

			// Install Agent Manager if it was not previously installed.
			$am_installed = AgentManager::checkInstalledService('am');
			if (!$am_installed['file_status']) {

				$am_was_installed = AgentManager::amInstall();

				if (!$am_was_installed) {
					Option::setOptions(['am_installed' => FALSE]);
				}
			}

		}
	}

	/**
	 * Handles all the AJAX plugin's requests.
	 *
	 * @return void
	 */
	function ajax() {

		if (Helper::postRequest('ajax_action') != NULL && \JSession::checkToken('get')) {

			Ajax::activation();
			Ajax::agentsInstallation();
			Ajax::reinstallAgents();
			Ajax::chart();
			Ajax::wafDateFilter();
			Ajax::ignorePorts();
			Ajax::lazyLoad();
			Ajax::antivirus();
			Ajax::changeThemeMode();
			Ajax::userTimeZone();
			Ajax::quarantine();
			Ajax::reports();
			Ajax::settings();
			Ajax::remove();
			Ajax::logout();
			Ajax::popup();
			exit();
		}

		echo json_encode([
				'success' => false,
				'error' => 'invalid ajax request',
				'notifications' => Ajax::notifications(),
		]);
		exit();
	}

	/**
	 * Activation page.
	 *
	 * @return void
	 */
	private function activation() {

		$build[] = [
				'variables' => [
						'notifications' => Helper::getNotifications(),
						'current_year' => date('Y'),
				],
				'template' => 'activation'
		];

		$template = new Template();
		echo $template->arrayRender($build);
	}

	/**
	 * Error page.
	 *
	 * @return void
	 */
	private static function error_page(){
		$template = new Template();
		$build[] = [
			'template' => 'error',
		];
		$page_content = $template->arrayRender($build);
		echo $template->baseTemplate($page_content);

	}

	/**
	 * Dashboard, main page.
	 *
	 * @return void
	 */
	private function dashboard() {
		
		$host = API::siteInfo();

		if (!isset($host['id']) or !$host['id']) {
			self::error_page();
			return;
		}

		$template = new Template();

		// Get data from WebTotem API.
		$data = API::getAllData($host['id']);

		if (empty($data)) {
			self::error_page();
			return;
		}

		// Reset session data.
		Option::setSessionOptions([
				'firewall_period' => NULL,
				'ram_period' => NULL,
				'cpu_period' => NULL,
		]);

		// Scoring block.
		$service_data = $data['scoring']['result'];
		$total_score = round($data['scoring']['score']);
		$score_grading = Helper::scoreGrading($total_score);
		$build[] = [
			'variables' => [
				"host_id" => $host['id'],
				"total_score" => $total_score . "%",
				"tested_on" => Helper::dateFormatter($data['scoring']['lastTest']['time']),
				"server_ip" => $service_data['ip'] ?: ' - ',
				"location" => Helper::getCountryName($service_data['country']) ?: ' - ',
				"is_higher_than" => $service_data['isHigherThan'] . '%',
				"grade" => $score_grading['grade'],
				"color" => $score_grading['color'],
			],
			'template' => 'score',
		];

		// Agents installing process.
		$agents_data = [
				'av' => $data['antivirus']['status'],
				'waf'  => $data['firewall']['status'],
		];

		$agents_statuses = Helper::getAgentsStatuses($agents_data);

		if (!$agents_statuses['option_statuses']['av'] or !$agents_statuses['option_statuses']['waf']) {

			$status = [
					'av' => $agents_statuses['process_statuses']['av'] == 'installed',
					'waf' => $agents_statuses['process_statuses']['waf'] == 'installed',
			];

			Option::setOptions([
					'av_installed' => $status['av'],
					'waf_installed' => $status['waf'],
			]);

			$build[] = [
					'variables' => [
							"process_status" => $agents_statuses['process_statuses'],
					],
					'template' => 'agents',
			];
		}


		// Firewall header.
		$build[] = [
			'variables' => [
				"title" => \JText::_('WTOTEM_FIREWALL_ACTIVITY'),
			],
			'template' => 'section_header',
		];

		$is_period_available = Helper::isPeriodAvailable($data['agentManager']['createdAt']);

		// Firewall stats.
		$service_data = (isset($data['firewall'])) ? $data['firewall'] : [];
		$chart = Helper::generateWafChart($service_data['chart']);
		$build[] = [
			'variables' => [
				"is_waf_training" => $data['agentManager'] && Helper::isWafTraining( $data['agentManager']['createdAt'] ),
				"is_period_available" => $is_period_available,
				"most_attacks" => Helper::getMostAttacksData($service_data['map']),
				"all_attacks" => $chart['count_attacks'],
				"blocking" => $chart['count_blocks'],
				"not_blocking" => (int) $chart['count_attacks'] - (int) $chart['count_blocks'],
			],
			'template' => 'firewall_stats',
		];

		// Firewall filter form
		$build[] = [
			'variables' => [
				"is_period_available" => $is_period_available,
			],
			'template' => 'waf_filter_form',
		];

		// Firewall blocks.
		$build[] = [
			'variables' => [
				"chart" => $chart['chart'],
				"logs"  => Helper::wafLogs($service_data['logs']['edges']),
				'host_name' => $host['name'],
			],
			'template' => 'firewall',
		];

		// Server Status header.
		$build[] = [
			'variables' => [
				"title" => \JText::_('WTOTEM_SERVER_RESOURCES'),
				"tooltip" => [
					'title' => \JText::_('WTOTEM_SERVER_RESOURCES'),
					'text' => \JText::_('WTOTEM_SERVER_RESOURCES_DESC'),
				],
			],
			'template' => 'section_header',
		];

		// Server Status RAM.
		$service_data = $data['serverStatus'];

		$build[] = [
			'variables' => [
				"is_period_available" => $is_period_available,
				"info" => $service_data['info'],
				"chart" => Helper::generateChart($service_data['ramChart']),
				'days' => 7,
			],
			'template' => 'server_status_ram',
		];

		// Server Status CPU.
		$build[] = [
			'variables' => [
				"is_period_available" => $is_period_available,
				"chart" => Helper::generateChart($service_data['cpuChart']),
			],

			'template' => 'server_status_cpu',
		];

		// Antivirus header.
		$build[] = [
			'variables' => [
				"title" => \JText::_('WTOTEM_ANTIVIRUS'),
			],
			'template' => 'section_header',
		];

		// Antivirus stats blocks.
		$antivirus_stats = $data['antivirus']['stats'];
		$build[] = [
			'variables' => [
				"changes"  => $antivirus_stats['changed'] ?: 0,
				"scanned"  => $antivirus_stats['scanned'] ?: 0,
				"deleted"  => $antivirus_stats['deleted'] ?: 0,
				"infected" => $antivirus_stats["infected"] ?: 0,
			],

			'template' => 'antivirus_stats',
		];


		// Monitoring header.
		$build[] = [
			'variables' => [
				"title" => \JText::_('WTOTEM_MONITORING'),
			],
			'template' => 'section_header',
		];

		// Monitoring blocks.
		$build[] = [
			'variables' => [
				"ssl"  => [
					'status' => Helper::getStatusData($data['ssl']['status']),
					'days_left' => Helper::daysLeft($data['ssl']['expiryDate']),
					'issue_date' => Helper::dateFormatter($data['ssl']['issueDate']),
					'expiry_date' => Helper::dateFormatter($data['ssl']['expiryDate']),
				],
				"availability"  => [
					'status' => Helper::getStatusData($data['availability']['status']),
					"percent" => $data['availability']['percent'],
					"response_time" => ceil($data['availability']['responseTime'] / 1000000) . ' ' . \JText::_('WTOTEM_MS'),
					"downtime" => ceil($data['availability']['downTime'] / 1000000) . ' ' . \JText::_('WTOTEM_MS'),
					"last_test" => Helper::dateFormatter($data['availability']['lastTest']['time']),
				],
				'reputation' => [
					"status" => Helper::getStatusData($data['reputation']['status']),
					"blacklists_entries" => Helper::blacklistsEntries(
						$data['reputation']['status'],
						$data['reputation']['virusList']),
					"info" => Helper::getReputationInfo($data['reputation']['status']),
					"last_test" => Helper::dateFormatter($data['reputation']['lastTest']['time']),
				],
			],
			'template' => 'monitoring',
		];

		$build[] = [
			'variables' => [
				"ports" => API::getAllPortsList($host['id']),
			],
			'template' => 'ports_form',
		];

		// Scanning header.
		$build[] = [
			'variables' => [
				"title" => \JText::_('WTOTEM_SCANNING'),
			],
			'template' => 'section_header',
		];

		$disc_usage_data = $data['serverStatus']['discUsage'];
		$disc_usage = [
			'total' => $disc_usage_data['total'],
			'free'  => $disc_usage_data['free'],
			'used'  => $disc_usage_data['total'] - $disc_usage_data['free'],
		];

		// Scanning blocks.
		$build[] = [
			'variables' => [
				"ports"  => [
					'status' => Helper::getStatusData($data['ports']['status']),
					"ip" => $data['ports']['ip'],
					"number" => count($data['ports']['tcp']),
					"tcp" => $data['ports']['tcp'],
					"ignore_ports" => $data['ports']['ignorePorts'],
					"last_test" => Helper::dateFormatter($data['ports']['lastTest']['time']),
				],
				"deface"  => [
					'status' => Helper::getStatusData($data['deface']['status']),
					"number" => $data['deface']['count'],
					"words" => !empty($data['deface']['words']) ? implode(",", $data['deface']['words']) : '',
					"last_test" => Helper::dateFormatter($data['deface']['lastTest']['time']),
				],
				"disc_usage" => $disc_usage,
				"disc_chart" => json_encode($disc_usage),
			],
			'template' => 'scanning',
		];

		$page_content = $template->arrayRender($build);
		echo $template->baseTemplate($page_content);
	}

	/** Firewall page.
	 *
	 * @return void
	 */
	private function firewall() {
		$host = API::siteInfo();

		if (!isset($host['id']) or !$host['id']) {
			self::error_page();
			return;
		}

		$template = new Template();

		// Get data from WebTotem API.
		if($cacheData = Cache::getdata('getFirewall', $host['id'])){
			$data = $cacheData['data'];
		} else {
			$data = API::getFirewall($host['id'], 10, NULL, 7);
			Cache::setData(['getFirewall' => $data], $host['id'], 1);
		}

		if (empty($data)) {
			self::error_page();
			return;
		}

		$service_data = $data['firewall'];

		// Reset session data.
		Option::setSessionOptions([
			'firewall_period' => NULL,
			'firewall_cursor' => $service_data['logs']['pageInfo']['endCursor'],
		]);


		// Start build array for rendering.
		// Firewall header.
		$build[] = [
			'variables' => [
				"title" => \JText::_('WTOTEM_FIREWALL_ACTIVITY'),
			],
			'template' => 'section_header',
		];

		// Attacks map blocks.
		// Get world_map json data
		$world_map_json = WTOTEM_MEDIA_URI . '/assets/js/world_map.json';
		$map_data = Helper::generateAttacksMapChart($service_data['map']);
		$is_period_available = Helper::isPeriodAvailable($data['agentManager']['createdAt']);

		$build[] = [
			'variables' => [
				"is_period_available"   => $is_period_available,
				"attacks_map" => $map_data,
				"world_map_json" => $world_map_json,
			],
			'template' => 'attacks_map',
		];

		// Firewall stats.
		$chart = Helper::generateWafChart($service_data['chart']);
		$build[] = [
			'variables' => [
				"is_waf_training" => isset( $data['agentManager']['createdAt'] ) && Helper::isWafTraining( $data['agentManager']['createdAt'] ),
				"is_period_available"   => $is_period_available,
				"all_attacks"   => $chart['count_attacks'],
				"blocking"    => $chart['count_blocks'],
				"not_blocking"  => $chart['count_attacks'] - $chart['count_blocks'],
				"most_attacks"  => Helper::getMostAttacksData($service_data['map']),
			],
			'template' => 'firewall_stats',
		];

		// Firewall filter form
		$build[] = [
			'template' => 'waf_filter_form',
		];

		// Firewall blocks.
		$build[] = [
			'variables' => [
				"chart" => $chart['chart'],
				"logs"  => Helper::wafLogs($service_data['logs']['edges']),
				'has_next_page' => $service_data['logs']['pageInfo']['hasNextPage'],
				'host_name' => $host['name'],
				'page'  => 'firewall',
			],
			'template' => 'firewall',
		];

		$page_content = $template->arrayRender($build);
		echo $template->baseTemplate($page_content);

	}

	/**
	 * Antivirus page.
	 *
	 * @return void
	 */
	private function antivirus() {

		$host = API::siteInfo();

		if (!isset($host['id']) or !$host['id']) {
			self::error_page();
			return;
		}

		$template = new Template();

		$params = [
			'host_id' => $host['id'],
			'limit' => 10,
			'cursor' => NULL,
			'days' => 365,
			'event' => FALSE,
			'permissions' => FALSE,
		];

		// Get data from WebTotem API.
		if($cacheData = Cache::getdata('getAntivirus', $host['id'])){
			$data = $cacheData['data'];
		} else {
			$data = API::getAntivirus($params);
			Cache::setData(['getAntivirus' => $data], $host['id']);
		}

		if (empty($data)) {
			self::error_page();
			return;
		}

		// Reset session data.
		Option::setSessionOptions([
			'antivirus_event' => NULL,
			'antivirus_permissions' => NULL,
			'antivirus_cursor' => $data['log']['pageInfo']['endCursor'],
		]);

		// Antivirus header.
		$build[] = [
			'variables' => [
				"title" => \JText::_('WTOTEM_ANTIVIRUS'),
			],
			'template' => 'section_header',
		];

		// Antivirus stats blocks.
		$stats = $data['stats'];
		$build[] = [
			'variables' => [
				'changes'  => $stats['changed'] ?: 0,
				'scanned'  => $stats['scanned'] ?: 0,
				'deleted'  => $stats['deleted'] ?: 0,
				'infected' => $stats["infected"] ?: 0,
				'page' => 'antivirus',
			],
			'template' => 'antivirus_stats',
		];

		// Quarantine logs blocks.
		$quarantine_logs = $data['quarantine'] ?: [];
		$quarantine_count = count($quarantine_logs);

		$build[] = [
			'variables' => [
				"logs"  => Helper::getQuarantineLogs($quarantine_logs) ?: [],
				"count"  => $quarantine_count,
			],
			'template' => 'quarantine',
		];

		// Antivirus filter form.
		$build[] = [
			'template' => 'antivirus_filter_form',
		];

		// Antivirus blocks.
		$build[] = [
			'variables' => [
				"logs" => Helper::getAntivirusLogs($data['log']['edges']),
				"has_next_page" => $data['log']['pageInfo']['hasNextPage'],
				'last_scan' => Helper::dateFormatter($data['lastTest']['time']),
			],

			'template' => 'antivirus',
		];

		$page_content = $template->arrayRender($build);
		echo $template->baseTemplate($page_content);
	}

	/**
	 * Settings page
	 *
	 * @return void
	 */
	private function settings() {

		$host = API::siteInfo();

		if (!isset($host['id']) or !$host['id']) {
			self::error_page();
			return;
		}

		$template = new Template();

		// Get data from WebTotem API.
		if($cacheData = Cache::getdata('getConfigs', $host['id'])){
			$configs_data = $cacheData['data'];
		} else {
			$configs_data = API::getConfigs($host['id']);
			Cache::setData(['getConfigs' => $configs_data], $host['id']);
		}

		$agents_statuses = API::getAgentsStatusesFromAPI($host['id']);

		if($cacheData = Cache::getdata('getIpLists', $host['id'])){
			$ip_list = $cacheData['data'];
		} else {
			$ip_list = API::getIpLists($host['id']);
			Cache::setData(['getIpLists' => $ip_list], $host['id']);
		}

		if($cacheData = Cache::getdata('getAllowUrlList', $host['id'])){
			$url_list = $cacheData['data'];
		} else {
			$url_list = API::getAllowUrlList($host['id']) ?: [];
			Cache::setData(['getAllowUrlList' => $url_list], $host['id']);
		}

		if($cacheData = Cache::getdata('getBlockedCountries', $host['id'])){
			$waf_data = $cacheData['data'];
		} else {
			$waf_data = API::getBlockedCountries($host['id']);
			Cache::setData(['getBlockedCountries' => $waf_data], $host['id']);
		}

		if (empty($configs_data) or
			empty($agents_statuses) or
			empty($ip_list)
		) {
			self::error_page();
			return;
		}

		// Settings form.
		$build[] = [
			'variables' => [
				'configs' => Helper::getConfigsData($configs_data, 'service'),
				'deny_list' => Helper::getIpList($ip_list['blackList'], 'ip_deny'),
				'allow_list' => Helper::getIpList($ip_list['whiteList'], 'ip_allow'),
				'url_list' => Helper::getUrlAllowList($url_list),
				'av_status' => Helper::getStatusData($agents_statuses['av']['status']),
				'waf_status' => Helper::getStatusData($agents_statuses['waf']['status']),
				'waf_settings' => Helper::getWafSettingData($ip_list['settings']),
				'blocked_countries_list' => json_encode($waf_data['blockedCountries']),
				'mock_attacks' => json_encode(Helper::getTreeMostAttacksData($waf_data['map'])),
			],

			'template' => 'settings_form',
		];

		$page_content = $template->arrayRender($build);
		echo $template->baseTemplate($page_content);
	}

	/**
	 * Reports page.
	 *
	 * @return void
	 */
	private function reports() {

		$host = API::siteInfo();

		if (!isset($host['id']) or !$host['id']) {
			self::error_page();
			return;
		}

		$template = new Template();

		// Get data from WebTotem API.
		if($cacheData = Cache::getdata('getAllReports', $host['id'])){
			$data = $cacheData['data'];
		} else {
			$data = API::getAllReports($host['id']);
			Cache::setData(['getAllReports' => $data], $host['id']);
		}

		if (empty($data)) {
			self::error_page();
			return;
		}

		Option::setSessionOptions([
			'reports_cursor' => $data['pageInfo']['endCursor'],
			'reports_m_cursor' => $data['pageInfo']['endCursor'],
		]);

		// Reports form.
		$build[] = [
			'template' => 'reports_form',
		];

		// Reports.
		$build[] = [
			'variables' => [
				"reports" => Helper::getReports($data['edges']),
				"has_next_page" => $data['pageInfo']['hasNextPage'],
			],
			'template' => 'reports',
		];

		$page_content = $template->arrayRender($build);
		echo $template->baseTemplate($page_content);
	}

	/**
	 * Information page.
	 *
	 * @return void
	 */
	private function documentation() {

		$template = new Template();

		$build[] = [
			'template' => 'help',
		];

		$page_content = $template->arrayRender($build);
		echo $template->baseTemplate($page_content);
	}

}
